@@ -38,6 +38,9 @@
 **
 ****************************************************************************/
 
+#include "android/androidutils_p.h"
+#include "android/localdevicebroadcastreceiver_p.h"
+#include "qbluetoothlocaldevice_p.h"
 #include <QtCore/QLoggingCategory>
 #include <QtCore/private/qjnihelpers_p.h>
 #include <QtAndroidExtras/QAndroidJniEnvironment>
@@ -45,9 +48,6 @@
 #include <QtBluetooth/QBluetoothLocalDevice>
 #include <QtBluetooth/QBluetoothAddress>
 
-#include "qbluetoothlocaldevice_p.h"
-#include "android/localdevicebroadcastreceiver_p.h"
-
 QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(QT_BT_ANDROID)
@@ -110,6 +110,12 @@ void QBluetoothLocalDevicePrivate::initialize(const QBluetoothAddress &address)
         return;
     }
 
+    if (!(ensureAndroidPermission(BluetoothPermission::Scan) &&
+          ensureAndroidPermission(BluetoothPermission::Connect))) {
+        qCWarning(QT_BT_ANDROID) <<  "Local device initialize() failed due to missing permissions";
+        return;
+    }
+
     obj = new QAndroidJniObject(adapter);
     if (!address.isNull()) {
         const QString localAddress
@@ -129,19 +135,26 @@ bool QBluetoothLocalDevicePrivate::isValid() const
 
 void QBluetoothLocalDevicePrivate::processHostModeChange(QBluetoothLocalDevice::HostMode newMode)
 {
-    if (!pendingHostModeTransition) {
-        // if not in transition -> pass data on
+    qCDebug(QT_BT_ANDROID) << "Processing host mode change:" << newMode
+                           << ", pending transition:" << pendingConnectableHostModeTransition;
+    if (!pendingConnectableHostModeTransition) {
+        // If host mode is not in transition -> pass data on
         emit q_ptr->hostModeStateChanged(newMode);
         return;
     }
 
+    // Host mode is in transition: check if the new mode is 'off' in which state
+    // we can enter the targeted 'Connectable' state
     if (isValid() && newMode == QBluetoothLocalDevice::HostPoweredOff) {
-        bool success = (bool)obj->callMethod<jboolean>("enable", "()Z");
-        if (!success)
+        const bool success = (bool)QAndroidJniObject::callStaticMethod<jboolean>(
+                    "org/qtproject/qt5/android/bluetooth/QtBluetoothBroadcastReceiver",
+                    "setEnabled");
+        if (!success) {
+            qCWarning(QT_BT_ANDROID) << "Transitioning Bluetooth from OFF to ON failed";
             emit q_ptr->error(QBluetoothLocalDevice::UnknownError);
+        }
     }
-
-    pendingHostModeTransition = false;
+    pendingConnectableHostModeTransition = false;
 }
 
 // Return -1 if address is not part of a pending pairing request
@@ -245,44 +258,87 @@ void QBluetoothLocalDevice::powerOn()
         return;
 
     if (d_ptr->adapter()) {
-        bool ret = (bool)d_ptr->adapter()->callMethod<jboolean>("enable", "()Z");
-        if (!ret)
+        bool success(false);
+        if (QtAndroidPrivate::androidSdkVersion() >= 31) {
+            success = (bool)QAndroidJniObject::callStaticMethod<jboolean>(
+                        "org/qtproject/qt5/android/bluetooth/QtBluetoothBroadcastReceiver",
+                        "setEnabled");
+        } else {
+            success = (bool)d_ptr->adapter()->callMethod<jboolean>("enable", "()Z");
+        }
+        if (!success) {
+            qCWarning(QT_BT_ANDROID) << "Enabling bluetooth failed";
             emit error(QBluetoothLocalDevice::UnknownError);
+        }
     }
 }
 
 void QBluetoothLocalDevice::setHostMode(QBluetoothLocalDevice::HostMode requestedMode)
 {
-    QBluetoothLocalDevice::HostMode mode = requestedMode;
+    QBluetoothLocalDevice::HostMode nextMode = requestedMode;
     if (requestedMode == HostDiscoverableLimitedInquiry)
-        mode = HostDiscoverable;
+        nextMode = HostDiscoverable;
 
-    if (mode == hostMode())
+    if (nextMode == hostMode())
         return;
 
-    if (mode == QBluetoothLocalDevice::HostPoweredOff) {
+    switch (nextMode) {
+
+    case QBluetoothLocalDevice::HostPoweredOff: {
         bool success = false;
-        if (d_ptr->adapter())
-            success = (bool)d_ptr->adapter()->callMethod<jboolean>("disable", "()Z");
-
-        if (!success)
-            emit error(QBluetoothLocalDevice::UnknownError);
-    } else if (mode == QBluetoothLocalDevice::HostConnectable) {
-        if (hostMode() == QBluetoothLocalDevice::HostDiscoverable) {
-            // cannot directly go from Discoverable to Connectable
-            // we need to go to disabled mode and enable once disabling came through
-
-            setHostMode(QBluetoothLocalDevice::HostPoweredOff);
-            d_ptr->pendingHostModeTransition = true;
-        } else {
-            QAndroidJniObject::callStaticMethod<void>(
-                "org/qtproject/qt5/android/bluetooth/QtBluetoothBroadcastReceiver",
-                "setConnectable");
+        if (d_ptr->adapter()) {
+            if (QtAndroidPrivate::androidSdkVersion() >= 31) {
+                success = (bool)QAndroidJniObject::callStaticMethod<jboolean>(
+                            "org/qtproject/qt5/android/bluetooth/QtBluetoothBroadcastReceiver",
+                            "setDisabled");
+            } else {
+                success = (bool)d_ptr->adapter()->callMethod<jboolean>("disable", "()Z");
+            }
         }
-    } else if (mode == QBluetoothLocalDevice::HostDiscoverable
-               || mode == QBluetoothLocalDevice::HostDiscoverableLimitedInquiry) {
-        QAndroidJniObject::callStaticMethod<void>(
-            "org/qtproject/qt5/android/bluetooth/QtBluetoothBroadcastReceiver", "setDiscoverable");
+        if (!success) {
+            qCWarning(QT_BT_ANDROID) << "Unable to power off the adapter";
+            emit error(QBluetoothLocalDevice::UnknownError);
+        }
+        break;
+    }
+
+    case QBluetoothLocalDevice::HostConnectable: {
+        if (hostMode() == QBluetoothLocalDevice::HostDiscoverable) {
+            // On Android 'Discoverable' is actually 'CONNECTABLE_DISCOVERABLE', and
+            // it seems we cannot go directly from "Discoverable" to "Connectable". Instead
+            // we need to go to disabled mode first and then to the 'Connectable' mode
+            setHostMode(QBluetoothLocalDevice::HostPoweredOff);
+            d_ptr->pendingConnectableHostModeTransition = true;
+        } else {
+            const bool success = (bool)QAndroidJniObject::callStaticMethod<jboolean>(
+                        "org/qtproject/qt5/android/bluetooth/QtBluetoothBroadcastReceiver",
+                        "setEnabled");
+            if (!success) {
+                qCWarning(QT_BT_ANDROID) << "Unable to enable the Bluetooth";
+                emit error(QBluetoothLocalDevice::UnknownError);
+            }
+        }
+        break;
+    }
+
+    case QBluetoothLocalDevice::HostDiscoverable: {
+        if (!ensureAndroidPermission(BluetoothPermission::Advertise)) {
+            qCWarning(QT_BT_ANDROID) << "Local device setHostMode() failed due to"
+                                        "missing permissions";
+            return;
+        }
+        const bool success = (bool)QAndroidJniObject::callStaticMethod<jboolean>(
+                    "org/qtproject/qt5/android/bluetooth/QtBluetoothBroadcastReceiver",
+                    "setDiscoverable");
+        if (!success) {
+            qCWarning(QT_BT_ANDROID) << "Unable to set Bluetooth as discoverable";
+            emit error(QBluetoothLocalDevice::UnknownError);
+        }
+        break;
+    }
+    default:
+        qCWarning(QT_BT_ANDROID) << "setHostMode() unsupported host mode:" << nextMode;
+        break;
     }
 }
 
@@ -308,6 +364,12 @@ QBluetoothLocalDevice::HostMode QBluetoothLocalDevice::hostMode() const
 
 QList<QBluetoothHostInfo> QBluetoothLocalDevice::allDevices()
 {
+    // As a static class function we need to ensure permissions here (in addition to initialize())
+    if (!ensureAndroidPermission(BluetoothPermission::Connect)) {
+        qCWarning(QT_BT_ANDROID) <<  "Local device allDevices() failed due to"
+                                     "missing permissions";
+        return {};
+    }
     // Android only supports max of one device (so far)
     QList<QBluetoothHostInfo> localDevices;
 
