@@ -52,6 +52,7 @@
 #include <windows.devices.enumeration.h>
 #include <windows.devices.bluetooth.h>
 #include <windows.foundation.collections.h>
+#include <windows.networking.h>
 #include <windows.storage.streams.h>
 #include <wrl.h>
 
@@ -81,22 +82,11 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT_WINRT)
 #define TYPE_STRING 37
 #define TYPE_SEQUENCE 53
 
-static QByteArray byteArrayFromBuffer(const ComPtr<IBuffer> &buffer, bool isWCharString = false)
+// Helper to reverse given uchar array
+static void reverseArray(uchar data[], size_t length)
 {
-    ComPtr<Windows::Storage::Streams::IBufferByteAccess> byteAccess;
-    HRESULT hr = buffer.As(&byteAccess);
-    Q_ASSERT_SUCCEEDED(hr);
-    char *data;
-    hr = byteAccess->Buffer(reinterpret_cast<byte **>(&data));
-    Q_ASSERT_SUCCEEDED(hr);
-    UINT32 size;
-    hr = buffer->get_Length(&size);
-    Q_ASSERT_SUCCEEDED(hr);
-    if (isWCharString) {
-        QString valueString = QString::fromUtf16(reinterpret_cast<ushort *>(data)).left(size / 2);
-        return valueString.toUtf8();
-    }
-    return QByteArray(data, size);
+    for (size_t i = length; i > length/2; i--)
+        std::swap(data[length - i], data[i - 1]);
 }
 
 class QWinRTBluetoothServiceDiscoveryWorker : public QObject
@@ -111,7 +101,6 @@ public:
 Q_SIGNALS:
     void serviceFound(quint64 deviceAddress, const QBluetoothServiceInfo &info);
     void scanFinished(quint64 deviceAddress);
-    void scanCanceled();
     void errorOccured();
 
 private:
@@ -226,6 +215,14 @@ void QWinRTBluetoothServiceDiscoveryWorker::processServiceSearchResult(quint64 a
         hr = service->get_ConnectionServiceName(name.GetAddressOf());
         Q_ASSERT_SUCCEEDED(hr);
         const QString serviceName = QString::fromWCharArray(WindowsGetStringRawBuffer(name.Get(), nullptr));
+        ComPtr<ABI::Windows::Networking::IHostName> host;
+        hr = service->get_ConnectionHostName(host.GetAddressOf());
+        Q_ASSERT_SUCCEEDED(hr);
+        HString hostName;
+        hr = host->get_RawName(hostName.GetAddressOf());
+        Q_ASSERT_SUCCEEDED(hr);
+        const QString qHostName = QString::fromWCharArray(WindowsGetStringRawBuffer(hostName.Get(),
+                                                                                    nullptr));
         ComPtr<IRfcommServiceId> id;
         hr = service->get_ServiceId(&id);
         Q_ASSERT_SUCCEEDED(hr);
@@ -235,6 +232,8 @@ void QWinRTBluetoothServiceDiscoveryWorker::processServiceSearchResult(quint64 a
         Q_ASSERT_SUCCEEDED(hr);
 
         QBluetoothServiceInfo info;
+        info.setAttribute(0xBEEF, QVariant(qHostName));
+        info.setAttribute(0xBEF0, QVariant(serviceName));
         info.setServiceName(serviceName);
         info.setServiceUuid(uuid);
         ComPtr<IAsyncOperation<IMapView<UINT32, IBuffer *> *>> op;
@@ -316,6 +315,8 @@ void QWinRTBluetoothServiceDiscoveryWorker::processServiceSearchResult(quint64 a
                 GUID value;
                 hr = dataReader->ReadGuid(&value);
                 Q_ASSERT_SUCCEEDED(hr);
+                // The latter 8 bytes are in reverse order
+                reverseArray(value.Data4, sizeof(value.Data4)/sizeof(value.Data4[0]));
                 const QBluetoothUuid uuid(value);
                 info.setAttribute(key, uuid);
                 qCDebug(QT_BT_WINRT) << "UUID" << uuid << "KEY" << hex << key << "TYPE" << dec << type << "UUID" << hex << uuid;
@@ -343,6 +344,17 @@ void QWinRTBluetoothServiceDiscoveryWorker::processServiceSearchResult(quint64 a
             }
             hr = iterator->MoveNext(&current);
         }
+        // Windows is only able to discover Rfcomm services but the according protocolDescriptor is
+        // not always set in the raw attribute map. If we encounter a service like that we should
+        // fill the protocol descriptor ourselves.
+        if (info.protocolDescriptor(QBluetoothUuid::Rfcomm).isEmpty()) {
+            QBluetoothServiceInfo::Sequence protocolDescriptorList;
+            QBluetoothServiceInfo::Sequence protocol;
+            protocol << QVariant::fromValue(QBluetoothUuid(QBluetoothUuid::Rfcomm))
+                     << QVariant::fromValue(0);
+            protocolDescriptorList.append(QVariant::fromValue(protocol));
+            info.setAttribute(QBluetoothServiceInfo::ProtocolDescriptorList, protocolDescriptorList);
+        }
         emit serviceFound(address, info);
     }
     emit scanFinished(address);
@@ -418,7 +430,8 @@ QBluetoothServiceInfo::Sequence QWinRTBluetoothServiceDiscoveryWorker::readSeque
             GUID b;
             hr = dataReader->ReadGuid(&b);
             Q_ASSERT_SUCCEEDED(hr);
-
+            // The latter 8 bytes are in reverse order
+            reverseArray(b.Data4, sizeof(b.Data4)/sizeof(b.Data4[0]));
             const QBluetoothUuid uuid(b);
             result.append(QVariant::fromValue(uuid));
             remainingLength -= sizeof(GUID);
@@ -490,7 +503,7 @@ QBluetoothServiceDiscoveryAgentPrivate::QBluetoothServiceDiscoveryAgentPrivate(
 
 QBluetoothServiceDiscoveryAgentPrivate::~QBluetoothServiceDiscoveryAgentPrivate()
 {
-    stop();
+    releaseWorker();
 }
 
 void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &address)
@@ -504,8 +517,6 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
         this, &QBluetoothServiceDiscoveryAgentPrivate::processFoundService, Qt::QueuedConnection);
     connect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanFinished,
         this, &QBluetoothServiceDiscoveryAgentPrivate::onScanFinished, Qt::QueuedConnection);
-    connect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanCanceled,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::onScanCanceled, Qt::QueuedConnection);
     connect(worker, &QWinRTBluetoothServiceDiscoveryWorker::errorOccured,
         this, &QBluetoothServiceDiscoveryAgentPrivate::onError, Qt::QueuedConnection);
     worker->start();
@@ -513,20 +524,9 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
 
 void QBluetoothServiceDiscoveryAgentPrivate::stop()
 {
-    if (!worker)
-        return;
-
-    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::serviceFound,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::processFoundService);
-    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanFinished,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::onScanFinished);
-    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanCanceled,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::onScanCanceled);
-    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::errorOccured,
-        this, &QBluetoothServiceDiscoveryAgentPrivate::onError);
-    // mWorker will delete itself as soon as it is done with its discovery
-    worker = nullptr;
-    setDiscoveryState(Inactive);
+    releaseWorker();
+    Q_Q(QBluetoothServiceDiscoveryAgent);
+    emit q->canceled();
 }
 
 void QBluetoothServiceDiscoveryAgentPrivate::processFoundService(quint64 deviceAddress, const QBluetoothServiceInfo &info)
@@ -575,26 +575,12 @@ void QBluetoothServiceDiscoveryAgentPrivate::processFoundService(quint64 deviceA
 
 void QBluetoothServiceDiscoveryAgentPrivate::onScanFinished(quint64 deviceAddress)
 {
-    Q_Q(QBluetoothServiceDiscoveryAgent);
-    bool deviceFound;
-    for (const QBluetoothDeviceInfo &deviceInfo : qAsConst(discoveredDevices)) {
-        if (deviceInfo.address().toUInt64() == deviceAddress) {
-            deviceFound = true;
-            discoveredDevices.removeOne(deviceInfo);
-            if (discoveredDevices.isEmpty())
-                setDiscoveryState(Inactive);
-            break;
-        }
-    }
-    Q_ASSERT(deviceFound);
-    stop();
-    emit q->finished();
-}
-
-void QBluetoothServiceDiscoveryAgentPrivate::onScanCanceled()
-{
-    Q_Q(QBluetoothServiceDiscoveryAgent);
-    emit q->canceled();
+    // The scan for a device's services has finished. Disconnect the
+    // worker and call the baseclass function which starts the scan for
+    // the next device if there are any unscanned devices left (or finishes
+    // the scan if none left)
+    releaseWorker();
+    _q_serviceDiscoveryFinished();
 }
 
 void QBluetoothServiceDiscoveryAgentPrivate::onError()
@@ -606,6 +592,21 @@ void QBluetoothServiceDiscoveryAgentPrivate::onError()
     emit q->error(error);
 }
 
+void QBluetoothServiceDiscoveryAgentPrivate::releaseWorker()
+{
+    if (!worker)
+        return;
+
+    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::serviceFound,
+        this, &QBluetoothServiceDiscoveryAgentPrivate::processFoundService);
+    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::scanFinished,
+        this, &QBluetoothServiceDiscoveryAgentPrivate::onScanFinished);
+    disconnect(worker, &QWinRTBluetoothServiceDiscoveryWorker::errorOccured,
+        this, &QBluetoothServiceDiscoveryAgentPrivate::onError);
+    // mWorker will delete itself as soon as it is done with its discovery
+    worker = nullptr;
+}
+
 QT_END_NAMESPACE
 
 #include <qbluetoothservicediscoveryagent_winrt.moc>
