@@ -47,6 +47,10 @@
 #include "qbluetoothsocket_android_p.h"
 #elif defined(QT_WINRT_BLUETOOTH)
 #include "qbluetoothsocket_winrt_p.h"
+#elif defined(QT_WIN_BLUETOOTH)
+#include "qbluetoothsocket_win_p.h"
+#elif defined(QT_OSX_BLUETOOTH)
+#include "qbluetoothsocket_osx_p.h"
 #else
 #include "qbluetoothsocket_dummy_p.h"
 #endif
@@ -91,6 +95,11 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT)
 
     On iOS, this class cannot be used because the platform does not expose
     an API which may permit access to QBluetoothSocket related features.
+
+    \note On macOS Monterey (12) the socket data flow is paused when a
+    modal dialogue is executing, or an event tracking mode is entered (for
+    example by long-pressing a Window close button). This may change in the
+    future releases of macOS.
 */
 
 /*!
@@ -267,6 +276,10 @@ static QBluetoothSocketBasePrivate *createSocketPrivate()
     return new QBluetoothSocketPrivateAndroid();
 #elif defined(QT_WINRT_BLUETOOTH)
     return new QBluetoothSocketPrivateWinRT();
+#elif defined(QT_WIN_BLUETOOTH)
+    return new QBluetoothSocketPrivateWin();
+#elif defined(QT_OSX_BLUETOOTH)
+    return new QBluetoothSocketPrivate();
 #else
     return new QBluetoothSocketPrivateDummy();
 #endif
@@ -513,6 +526,9 @@ QString QBluetoothSocket::errorString() const
 */
 void QBluetoothSocket::setPreferredSecurityFlags(QBluetooth::SecurityFlags flags)
 {
+#ifdef QT_OSX_BLUETOOTH
+    return; // not supported on macOS.
+#endif
     Q_D(QBluetoothSocketBase);
     if (d->secFlags != flags)
         d->secFlags = flags;
@@ -534,8 +550,13 @@ void QBluetoothSocket::setPreferredSecurityFlags(QBluetooth::SecurityFlags flags
 */
 QBluetooth::SecurityFlags QBluetoothSocket::preferredSecurityFlags() const
 {
+#if QT_OSX_BLUETOOTH
+    // not supported on macOS - platform always uses encryption
+    return QBluetooth::Secure;
+#else
     Q_D(const QBluetoothSocketBase);
     return d->secFlags;
+#endif // QT_OSX_BLUETOOTH
 }
 
 /*!
@@ -559,6 +580,9 @@ void QBluetoothSocket::setSocketState(QBluetoothSocket::SocketState state)
         emit disconnected();
     }
     if(state == ListeningState){
+#ifdef QT_OSX_BLUETOOTH
+        qCWarning(QT_BT) << "listening socket is not supported by IOBluetooth";
+#endif
         // TODO: look at this, is this really correct?
         // if we're a listening socket we can't handle connects?
         if (d->readNotifier) {
@@ -574,7 +598,7 @@ void QBluetoothSocket::setSocketState(QBluetoothSocket::SocketState state)
 bool QBluetoothSocket::canReadLine() const
 {
     Q_D(const QBluetoothSocketBase);
-    return d->canReadLine();
+    return d->canReadLine() || QIODevice::canReadLine();
 }
 
 /*!
@@ -640,6 +664,13 @@ void QBluetoothSocket::serviceDiscovered(const QBluetoothServiceInfo &service)
         connectToService(service, d->openMode);
         d->discoveryAgent->deleteLater();
         d->discoveryAgent = nullptr;
+#ifdef QT_WINRT_BLUETOOTH
+    } else if (!service.attribute(0xBEEF).isNull()
+               && !service.attribute(0xBEF0).isNull()) {
+        connectToService(service, d->openMode);
+        d->discoveryAgent->deleteLater();
+        d->discoveryAgent = nullptr;
+#endif
     } else {
         qCDebug(QT_BT) << "Could not find port/psm for potential remote service";
     }
@@ -760,6 +791,9 @@ void QBluetoothSocket::close()
   Set the socket to use \a socketDescriptor with a type of \a socketType,
   which is in state, \a socketState, and mode, \a openMode.
 
+  The set socket descriptor is considered owned by the QBluetoothSocket
+  and may be e.g. closed once finished.
+
   Returns true on success
 */
 
@@ -843,6 +877,6 @@ QDebug operator<<(QDebug debug, QBluetoothSocket::SocketState state)
 }
 #endif
 
-#include "moc_qbluetoothsocket.cpp"
-
 QT_END_NAMESPACE
+
+#include "moc_qbluetoothsocket.cpp"
