@@ -54,10 +54,19 @@
 #include "qlowenergycontroller_bluez_p.h"
 #elif defined(QT_ANDROID_BLUETOOTH)
 #include "qlowenergycontroller_android_p.h"
+#include "android/androidutils_p.h"
 #elif defined(QT_WINRT_BLUETOOTH)
+#include "qtbluetoothglobal_p.h"
 #include "qlowenergycontroller_winrt_p.h"
+#if QT_CONFIG(winrt_btle_no_pairing)
+#include "qlowenergycontroller_winrt_new_p.h"
+#endif
+#elif defined(QT_WIN_BLUETOOTH)
+#include "qlowenergycontroller_win_p.h"
+#elif defined(Q_OS_DARWIN)
+#include "qlowenergycontroller_darwin_p.h"
 #else
-#include "qlowenergycontroller_p.h"
+#include "qlowenergycontroller_dummy_p.h"
 #endif
 
 #include <algorithm>
@@ -65,6 +74,10 @@
 QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(QT_BT)
+Q_DECLARE_LOGGING_CATEGORY(QT_BT_WINRT)
+#if defined(QT_ANDROID_BLUETOOTH)
+Q_DECLARE_LOGGING_CATEGORY(QT_BT_ANDROID)
+#endif
 
 /*!
     \class QLowEnergyController
@@ -152,6 +165,9 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT)
                                         This value was introduced by Qt 5.7.
     \value RemoteHostClosedError        The remote device closed the connection.
                                         This value was introduced by Qt 5.10.
+    \value AuthorizationError           The local Bluetooth device closed the connection due to
+                                        insufficient authorization.
+                                        This value was introduced by Qt 5.14.
 */
 
 /*!
@@ -200,7 +216,7 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT)
    \sa QLowEnergyController::createCentral()
    \sa QLowEnergyController::createPeripheral()
    \since 5.7
-   \note The peripheral role is currently only supported on Linux. In addition, handling the
+   \note The peripheral role is not supported on Windows. In addition on Linux, handling the
          "Signed Write" ATT command on the server side requires BlueZ 5 and kernel version 3.7
          or newer.
  */
@@ -310,7 +326,18 @@ static QLowEnergyControllerPrivate *privateController(QLowEnergyController::Role
     return new QLowEnergyControllerPrivateAndroid();
 #elif defined(QT_WINRT_BLUETOOTH)
     Q_UNUSED(role);
+#if QT_CONFIG(winrt_btle_no_pairing)
+    return createWinRTLowEnergyController();
+#else
+    qCDebug(QT_BT_WINRT) << "Using pre 15063 low energy controller";
     return new QLowEnergyControllerPrivateWinRT();
+#endif
+#elif defined(QT_WIN_BLUETOOTH)
+    Q_UNUSED(role);
+    return new QLowEnergyControllerPrivateWin32();
+#elif defined(Q_OS_DARWIN)
+    Q_UNUSED(role)
+    return new QLowEnergyControllerPrivateDarwin();
 #else
     Q_UNUSED(role);
     return new QLowEnergyControllerPrivateCommon();
@@ -334,6 +361,9 @@ QLowEnergyController::QLowEnergyController(
                             QObject *parent)
     : QObject(parent)
 {
+    // Note: a central created using this ctor is useless
+    // on Darwin - no way to use addresses when connecting.
+
     d_ptr = privateController(CentralRole);
 
     Q_D(QLowEnergyController);
@@ -363,11 +393,12 @@ QLowEnergyController::QLowEnergyController(
                             QObject *parent)
     : QObject(parent)
 {
-        d_ptr = privateController(CentralRole);
+    d_ptr = privateController(CentralRole);
 
     Q_D(QLowEnergyController);
     d->q_ptr = this;
     d->role = CentralRole;
+    d->deviceUuid = remoteDeviceInfo.deviceUuid();
     d->remoteDevice = remoteDeviceInfo.address();
     d->localAdapter = QBluetoothLocalDevice().address();
     d->addressType = QLowEnergyController::PublicAddress;
@@ -396,6 +427,8 @@ QLowEnergyController::QLowEnergyController(
                             QObject *parent)
     : QObject(parent)
 {
+    // Note: a central create using this ctor is useless on
+    // Darwin (CoreBluetooth does not work with addresses).
     d_ptr = privateController(CentralRole);
 
     Q_D(QLowEnergyController);
@@ -422,6 +455,29 @@ QLowEnergyController *QLowEnergyController::createCentral(const QBluetoothDevice
     return new QLowEnergyController(remoteDevice, parent);
 }
 
+/*!
+    Returns a new instance of this class with \a parent.
+
+    The \a remoteDevice must contain the address of the remote Bluetooth Low
+    Energy device to which this object should attempt to connect later on.
+
+    The connection is established via \a localDevice. If \a localDevice is invalid,
+    the local default device is automatically selected. If \a localDevice specifies
+    a local device that is not a local Bluetooth adapter, \l error() is set to
+    \l InvalidBluetoothAdapterError once \l connectToDevice() is called.
+
+    Note that specifying the local device to be used for the connection is only
+    possible when using BlueZ. All other platforms do not support this feature.
+
+    \since 5.14
+ */
+QLowEnergyController *QLowEnergyController::createCentral(const QBluetoothAddress &remoteDevice,
+                                                          const QBluetoothAddress &localDevice,
+                                                          QObject *parent)
+{
+    return new QLowEnergyController(remoteDevice, localDevice, parent);
+}
+
 
 /*!
    Returns a new object of this class that is in the \l PeripheralRole and has the
@@ -501,7 +557,7 @@ QBluetoothAddress QLowEnergyController::remoteAddress() const
  */
 QBluetoothUuid QLowEnergyController::remoteDeviceUuid() const
 {
-    return  QBluetoothUuid();
+    return d_ptr->deviceUuid;
 }
 
 /*!
@@ -583,6 +639,7 @@ void QLowEnergyController::connectToDevice()
     }
 
     if (!d->isValidLocalAdapter()) {
+        qCWarning(QT_BT) << "connectToDevice() LE controller has invalid adapter";
         d->setError(QLowEnergyController::InvalidBluetoothAdapterError);
         return;
     }
@@ -790,6 +847,13 @@ QLowEnergyService *QLowEnergyController::addService(const QLowEnergyServiceData
         return nullptr;
     }
 
+#if defined(QT_ANDROID_BLUETOOTH)
+    if (!ensureAndroidPermission(BluetoothPermission::Connect)) {
+        qCWarning(QT_BT_ANDROID) << "addService() failed due to missing permissions";
+        return nullptr;
+    }
+#endif
+
     Q_D(QLowEnergyController);
     QLowEnergyService *newService = d->addServiceHelper(service);
     if (newService)
@@ -864,3 +928,5 @@ QLowEnergyController::Role QLowEnergyController::role() const
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qlowenergycontroller.cpp"
